import {QueueingSubject}from'queueing-subject';import {VSBuffer}from'@vscode-alt/monaco-editor/esm/vs/base/common/buffer.js';import {Emitter}from'@vscode-alt/monaco-editor/esm/vs/base/common/event.js';import makeWebSocketObservable,{normalClosureMessage}from'./rxjs-websockets.js';import {switchMap,share}from'rxjs';class BrowserRemoteRPC {
    // this subject queues as necessary to ensure every message is delivered
    input$;
    _localHostId;
    _onMessage = new Emitter();
    constructor(hostId = 2 /* HostIdentifier.BrowserExtHost */) {
        this._localHostId = hostId;
        console.clear();
        this.input$ = new QueueingSubject();
    }
    onMessage(event) {
        return this._onMessage.event(event);
    }
    connect(url, makeWebSocket) {
        const socket$ = makeWebSocketObservable(url, {
            protocols: '',
            makeWebSocket,
        });
        const messages$ = socket$.pipe(
        // the observable produces a value once the websocket has been opened
        switchMap((getResponses) => {
            console.log('websocket opened');
            return getResponses(this.input$);
        }), share());
        messages$.subscribe((message) => {
            const buff = new Uint8Array(message);
            // @sri in future version of @vscode-alt/monaco-editor we don't need  to do add `Buffer.from`
            // noticed `Buffer` in browser will be polyfilled by webpack
            const vsBuffer = VSBuffer.wrap(Buffer.from(buff.buffer));
            this._onMessage.fire(vsBuffer);
        }, (error) => {
            const { message } = error;
            if (message === normalClosureMessage) {
                console.log('server closed the websocket connection normally');
            }
            else {
                console.log('socket was disconnected due to error:', message);
            }
        });
    }
    send(buffer, hostIdentifier) {
        // If the websocket is not connected then the QueueingSubject will ensure
        // that messages are queued and delivered when the websocket reconnects.
        // A regular Subject can be used to discard messages sent when the websocket
        // is disconnected.
        const headerBuf = VSBuffer.alloc(2);
        // add destination host id
        headerBuf.writeUInt8(hostIdentifier, 0);
        // add sender host id
        headerBuf.writeUInt8(this._localHostId, 1);
        const cocnatBuf = VSBuffer.concat([headerBuf, buffer]);
        // converts to ArrayBuffer and sends data
        this.input$.next(cocnatBuf.buffer);
    }
}export{BrowserRemoteRPC};