import { UriComponents } from '@vscode-alt/monaco-editor/esm/vs/base/common/uri';
import { ConfigurationScope, IConfigurationModel, IConfigurationData } from '@adminide-stack/core';
import * as cdeops from 'cdeops';
import { Selection } from '@vscode-alt/monaco-editor/esm/vs/editor/common/core/selection';
import { IRange } from '@vscode-alt/monaco-editor/esm/vs/editor/common/core/range';
import * as modes from '@vscode-alt/monaco-editor/esm/vs/editor/common/modes';
import { CharacterPair, CommentRule, EnterAction } from '@vscode-alt/monaco-editor/esm/vs/editor/common/modes/languageConfiguration';
import { IMarkerData } from '@vscode-alt/monaco-editor/esm/vs/platform/markers/common/markers';
import { IModelChangedEvent } from './temporary-types';
import * as editorCommonAdditions from './editor-missing-types';
import { IMarkdownString } from '@vscode-alt/monaco-editor/esm/vs/base/common/htmlContent';
import { ISingleEditOperation } from '@vscode-alt/monaco-editor/esm/vs/editor/common/model';
type EditorViewColumn = editorCommonAdditions.EditorPosition;
export interface IRawColorInfo {
    color: [number, number, number, number];
    range: IRange;
}
export declare class IdObject {
    _id?: number;
    private static _n;
    static mixin<T extends object>(object: T): T & IdObject;
}
export declare const enum ISuggestDataDtoField {
    label = "a",
    kind = "b",
    detail = "c",
    documentation = "d",
    sortText = "e",
    filterText = "f",
    preselect = "g",
    insertText = "h",
    insertTextRules = "i",
    range = "j",
    commitCharacters = "k",
    additionalTextEdits = "l",
    command = "m",
    kindModifier = "n",
    label2 = "o"
}
export interface ISuggestDataDto {
    [ISuggestDataDtoField.label]: string;
    [ISuggestDataDtoField.label2]?: string | modes.CompletionItemLabel;
    [ISuggestDataDtoField.kind]?: modes.CompletionItemKind;
    [ISuggestDataDtoField.detail]?: string;
    [ISuggestDataDtoField.documentation]?: string | IMarkdownString;
    [ISuggestDataDtoField.sortText]?: string;
    [ISuggestDataDtoField.filterText]?: string;
    [ISuggestDataDtoField.preselect]?: true;
    [ISuggestDataDtoField.insertText]?: string;
    [ISuggestDataDtoField.insertTextRules]?: modes.CompletionItemInsertTextRule;
    [ISuggestDataDtoField.range]?: IRange | {
        insert: IRange;
        replace: IRange;
    };
    [ISuggestDataDtoField.commitCharacters]?: string[];
    [ISuggestDataDtoField.additionalTextEdits]?: ISingleEditOperation[];
    [ISuggestDataDtoField.command]?: modes.Command;
    [ISuggestDataDtoField.kindModifier]?: modes.CompletionItemTag[];
    x?: ChainedCacheId;
}
export type CacheId = number;
export type ChainedCacheId = [CacheId, CacheId];
export declare const enum ISuggestResultDtoField {
    defaultRanges = "a",
    completions = "b",
    isIncomplete = "c",
    duration = "d"
}
export interface SuggestResultDto extends IdObject {
    [ISuggestResultDtoField.defaultRanges]: {
        insert: IRange;
        replace: IRange;
    };
    [ISuggestResultDtoField.completions]: ISuggestDataDto[];
    [ISuggestResultDtoField.isIncomplete]: undefined | true;
    [ISuggestResultDtoField.duration]: number;
    x?: number;
}
export interface WorkspaceSymbolDto extends IdObject {
    name: string;
    containerName?: string;
    kind: modes.SymbolKind;
    location: LocationDto;
}
export interface WorkspaceSymbolsDto extends IdObject {
    symbols: WorkspaceSymbolDto[];
}
export interface ResourceFileEditDto {
    oldUri: UriComponents;
    newUri: UriComponents;
}
export interface ResourceTextEditDto {
    resource: UriComponents;
    modelVersionId?: number;
    edits: modes.TextEdit[];
}
export interface WorkspaceEditDto {
    edits: (ResourceFileEditDto | ResourceTextEditDto)[];
    rejectReason?: string;
}
export declare function reviveWorkspaceEditDto(data: WorkspaceEditDto): modes.WorkspaceEdit;
export type CommandDto = ObjectIdentifier & modes.Command;
export interface CodeActionDto {
    title: string;
    edit?: WorkspaceEditDto;
    diagnostics?: IMarkerData[];
    command?: modes.Command;
    kind?: string;
}
export interface ExtHostDocumentSaveParticipantShape {
}
export interface ITextEditorPositionData {
    [id: string]: EditorViewColumn;
}
export interface ISelectionChangeEvent {
    selections: Selection[];
    source?: string;
}
export interface ExtHostDocumentContentProvidersShape {
    $provideTextDocumentContent(handle: number, uri: UriComponents): Promise<string>;
}
export interface IModelAddedData {
    uri: UriComponents;
    versionId: number;
    lines: string[];
    EOL: string;
    modeId: string;
    isDirty: boolean;
}
export interface ExtHostDocumentsShape {
    $acceptModelModeChanged(strURL: UriComponents, oldModeId: string, newModeId: string): void;
    $acceptModelSaved(strURL: UriComponents): void;
    $acceptDirtyStateChanged(strURL: UriComponents, isDirty: boolean): void;
    $acceptModelChanged(strURL: UriComponents, e: IModelChangedEvent, isDirty: boolean): void;
}
export interface IDocumentContentProvidersShape {
    $registerTextContentProvider(handle: number, scheme: string): void;
    $unregisterTextContentProvider(handle: number): void;
    $onVirtualDocumentChange(uri: UriComponents, value: string): void;
}
export interface IDocumentsShape {
    $tryCreateDocument(options?: {
        language?: string;
        content?: string;
    }): Promise<UriComponents>;
    $tryOpenDocument(uri: UriComponents): Promise<void>;
    $trySaveDocument(uri: UriComponents): Promise<boolean>;
}
export interface IUndoStopOptions {
    undoStopBefore: boolean;
    undoStopAfter: boolean;
}
export interface ITextDocumentShowOptions {
    position?: EditorViewColumn;
    preserveFocus?: boolean;
    pinned?: boolean;
    selection?: IRange;
}
/**
 * We prefix anything that runs on the main thread as `ClientThread` and the one that
 * runs on the extension host as `WebExtHost`. The `WebExtHost` services usually runs on
 * a webworker in browser.
 */
export interface IStaticOrganizationData {
    id: string;
    name: string;
    configuration?: UriComponents | null;
    isUntitled?: boolean | null;
}
export interface IOrganizationData extends IStaticOrganizationData {
    resources?: {
        uri: UriComponents;
        name: string;
        index: number;
    }[];
}
export interface DefinitionLinkDto {
    originSelectionRange?: IRange;
    uri: UriComponents;
    range: IRange;
    targetSelectionRange?: IRange;
}
export declare namespace ObjectIdentifier {
    const name = "$ident";
    function mixin<T>(obj: T, id: number): T & ObjectIdentifier;
    function of(obj: any): number;
}
export interface ObjectIdentifier {
    $ident: number;
}
export interface IOrganizationConfigurationChangeEventData {
    changedConfiguration: IConfigurationModel;
    changedConfigurationByResource: {
        [folder: string]: IConfigurationModel;
    };
}
export interface IConfigurationInitData extends IConfigurationData {
    configurationScopes: {
        [key: string]: ConfigurationScope;
    };
}
export interface ISerializedDocumentFilter {
    $serialized: true;
    language?: string;
    scheme?: string;
    pattern?: cdeops.GlobPattern;
    exclusive?: boolean;
}
export interface LocationDto {
    uri: UriComponents;
    range: IRange;
}
export interface LinkDto extends ObjectIdentifier {
    range: IRange;
    url?: string | UriComponents;
}
export interface ISerializedSignatureHelpProviderMetadata {
    readonly triggerCharacters: ReadonlyArray<string>;
    readonly retriggerCharacters: ReadonlyArray<string>;
}
export interface ISerializedRegExp {
    pattern: string;
    flags?: string;
}
export interface ISerializedIndentationRule {
    decreaseIndentPattern: ISerializedRegExp;
    increaseIndentPattern: ISerializedRegExp;
    indentNextLinePattern?: ISerializedRegExp;
    unIndentedLinePattern?: ISerializedRegExp;
}
export interface ISerializedOnEnterRule {
    beforeText: ISerializedRegExp;
    afterText?: ISerializedRegExp;
    oneLineAboveText?: ISerializedRegExp;
    action: EnterAction;
}
export interface ISerializedLanguageConfiguration {
    comments?: CommentRule;
    brackets?: CharacterPair[];
    wordPattern?: ISerializedRegExp;
    indentationRules?: ISerializedIndentationRule;
    onEnterRules?: ISerializedOnEnterRule[];
    __electricCharacterSupport?: {
        brackets?: any;
        docComment?: {
            scope: string;
            open: string;
            lineStart: string;
            close?: string;
        };
    };
    __characterPairSupport?: {
        autoClosingPairs: {
            open: string;
            close: string;
            notIn?: string[];
        }[];
    };
}
export interface ILanguageFeaturesShape {
    $unregister(handle: number): void;
    $registerOutlineSupport(handle: number, selector: ISerializedDocumentFilter[], extensionId: string): void;
    $registerCodeLensSupport(handle: number, selector: ISerializedDocumentFilter[], eventHandle: number): void;
    $registerCodeInsetSupport(handle: number, selector: ISerializedDocumentFilter[], eventHandle: number | undefined): void;
    $emitCodeLensEvent(eventHandle: number, event?: any): void;
    $registerDeclarationSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerDefinitionSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerImplementationSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerTypeDefinitionSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerHoverProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerDocumentHighlightProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerReferenceSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerQuickFixSupport(handle: number, selector: ISerializedDocumentFilter[], supportedKinds?: string[]): void;
    $registerDocumentFormattingSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerCompletionItemProvider(handle: number, selector: ISerializedDocumentFilter[], eventHandle: number): void;
    $registerRangeFormattingSupport(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerOnTypeFormattingSupport(handle: number, selector: ISerializedDocumentFilter[], autoFormatTriggerCharacters: string[]): void;
    $registerNavigateTypeSupport(handle: number): void;
    $registerRenameSupport(handle: number, selector: ISerializedDocumentFilter[], supportsResolveInitialValues: boolean): void;
    $registerSuggestSupport(handle: number, selector: ISerializedDocumentFilter[], triggerCharacters: string[], supportsResolveDetails: boolean): void;
    $registerSignatureHelpProvider(handle: number, selector: ISerializedDocumentFilter[], metadata: ISerializedSignatureHelpProviderMetadata): void;
    $registerDocumentLinkProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerDocumentColorProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerFoldingRangeProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerSelectionRangeProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $registerCallHierarchyProvider(handle: number, selector: ISerializedDocumentFilter[]): void;
    $setLanguageConfiguration(handle: number, languageId: string, configuration: ISerializedLanguageConfiguration): void;
}
export {};
